package proptypes;

import haxe.macro.Expr;
import haxe.macro.Type;
import haxe.macro.Context;
import haxe.macro.ComplexTypeTools;
import haxe.macro.TypeTools;

class PropTypesToType {
	public static function buildComponent():Array<Field> {
		var fields = Context.getBuildFields();

		var propTypes = getVarExpr(fields, 'propTypes');

		if (propTypes != null) {
			var propsFields = propTypes.map(function(fieldDecl) {
				return parsePropTypeField(fieldDecl.field, fieldDecl.expr);
			});

			// trace(propsFields);
			var propsType:AnonType = {fields: propsFields, status: AClosed};
			trace(propsType);
		}

		return fields;
	}

	static function getVarExpr(fields:Array<Field>, varName:String):Array<{field:String, expr:Expr}> {
		return fields
			.filter(function(field) return field.name == varName)
			.map(function(field) {
				return switch(field.kind) {
					case FVar(_, {expr: EObjectDecl(varFields)}):
						varFields;
					default: null;
				};
			})
			.shift();
	}

	static function parsePropTypeField(field:String, expr:Expr) {
		var isOptional:Bool = false;
		var field:ClassField = {
			isPublic: true,
			kind: FVar(AccNormal, AccNo),
			name: field,
			overloads: null,
			params: null,
			pos: Context.currentPos(),
			type: null,
			expr: null,
			meta: cast new Array<MetadataEntry>(),
			doc: null
		};

		switch (expr) {
			case macro ReactPropTypes.$propType,
			macro react.ReactPropTypes.$propType:
				isOptional = true;
				field.type = ComplexTypeTools.toType(getTypeFromPropType(propType));

			case macro ReactPropTypes.$propType.isRequired,
			macro react.ReactPropTypes.$propType.isRequired:
				field.type = ComplexTypeTools.toType(getTypeFromPropType(propType));

			case macro ReactPropTypes.arrayOf($arr),
			macro react.ReactPropTypes.arrayOf($arr):
				isOptional = true;

			case macro ReactPropTypes.arrayOf($arr).isRequired,
			macro react.ReactPropTypes.arrayOf($arr).isRequired:

			default:
				trace(expr);
		}

		if (isOptional)
			field.meta = cast [{name: ":optional", params: [], pos: Context.currentPos()}];

		return field;
	}

	static function getTypeFromPropType(propType:String):ComplexType {
		return switch (propType) {
			case "bool": macro :Bool;
			case "array": macro :Array<Dynamic>;
			case "number": macro :Float;
			case "string": macro :String;
			case "element": macro :react.ReactComponent.ReactElement;

			default: macro :Dynamic;
		}
	}
}
